/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il      */
/* License: GNU General Public License                              */

/*
 * $Id: module_interface.h,v 1.9 2003/01/27 17:20:42 mulix Exp $
 */

#ifndef MODULE_INTERFACE_H_
#define MODULE_INTERFACE_H_

#include <asm/ioctl.h>

/* 
 * defines used for user space communication with the module. 
 */

/* we use this magic cookie in the 'magic' field in every request */
/* packet going from userspace to kernelspace and vice versa. */ 
/* overkill? you betcha. but it will come in handy debugging and */ 
/* when we want to send these over the network */ 
#define DEV_FILE_MAGIC_COOKIE ((unsigned int)0xc0debabe)

/* we use this magic cookie for returning a 'get rules' variable */ 
/* length packet */ 
#define GET_RULES_MAGIC_COOKIE 0xcbad0123

/* in each reply the kernel returns to userspace, the 'cmd' field */ 
/* should be or'd with REPLY */ 
#define REPLY 0x80000000

/* the sysctl ID of syscalltrack */ 
#define CTL_TRACKER			                8


#define	CTL_TRACKER_CMD_ADD_RULE		        1
#define	CTL_TRACKER_CMD_DEL_RULE		        2
#define	CTL_TRACKER_CMD_ENABLE_RULE		        3
#define	CTL_TRACKER_CMD_DISABLE_RULE	                4
#define	CTL_TRACKER_CMD_DEL_ALL_RULES_FOR_SYSCALL	5
#define CTL_TRACKER_CMD_DEL_ALL_MODULE_RULES            6
#define CTL_TRACKER_CMD_PRINT_ALL_RULES                 7
#define CTL_TRACKER_CMD_GET_ALL_RULES                   8              
#define CTL_TRACKER_CMD_GET_ALL_RULE_COUNT              9


#define	CTL_TRACKER_SUCCESS	     0       /* the operation was succesfull.  */
#define CTL_TRACKER_EINVAL	     -EINVAL /* the 'request packet' is invalid */
#define	CTL_TRACKER_ENOMEM	     -ENOMEM /* the kernel could not allocate memory */
#define	CTL_TRACKER_EINTR	     -EINTR  /* performing the request was interrupted - */
                                             /* the client should try again. */
#define CTL_TRACKER_ENOSYS	     -ENOSYS /* no such syscall (invalid syscall_id parameter) */
#define CTL_TRACKER_ESYSUNSUP        -ENXIO  /* this syscall is not supported at the moment */
#define CTL_TRACKER_ENOENT	     -ENOENT /* no such tracking rule */
#define CTL_TRACKER_EBADTRACKINGRULE -EILSEQ /* the tracking rule could not be deserialized */


#define CTL_IOC_DROP_OLD          _IO(CTL_TRACKER, 0)
#define CTL_IOC_DROP_NEW          _IO(CTL_TRACKER, 1)
#define CTL_IOC_FLUSH             _IO(CTL_TRACKER, 2)
#define CTL_IOC_ENABLE_DEV        _IO(CTL_TRACKER, 3)
#define CTL_IOC_DISABLE_DEV       _IO(CTL_TRACKER, 4)
#define CTL_IOC_RESIZE            _IOW(CTL_TRACKER, 5, size_t)
#define CTL_IOC_BEFORE_LOG_FORMAT _IOW(CTL_TRACKER, 6, char*)
#define CTL_IOC_AFTER_LOG_FORMAT  _IOW(CTL_TRACKER, 7, char*)
#define CTL_IOC_MAX_REC_SIZE      _IOW(CTL_TRACKER, 8, unsigned int)

/* the requests exchanged between userspace and kernelspace have this */ 
/* format. note - the data field is actually *inline* in the request, */
/* so when reading it from the device file, you can't use this structure */ 
/* "as is" to read from the device, although that's a bad idea anyway */ 
/* (think field ordering and padding */
typedef struct dev_request_type {
	/* prefix */ 
	unsigned int size;    /* packet size */
	unsigned int id;      /* packet id */ 
	unsigned int cmd;     /* packet command, possibly or'd with REPLY */ 

	/* data */ 
	int result;           /* operation result - one of the error code above */ 
	unsigned int data_sz; /* data size */ 
	unsigned char* data;  /* pointer to data */ 
	
	/* sufix */ 
	unsigned int magic;            /* magic people, voodoo people*/ 
} dev_req_t; 

/* debug things */ 

#define DEB_NONE          (0UL)       /* nothing */ 
#define DEB_INFO          (1UL << 0)  /* misc */ 
#define DEB_DEV_FILE      (1UL << 1)  /* device file */ 
#define DEB_SYSCTL        (1UL << 2)  /* sysctl */ 
#define DEB_RULES         (1UL << 3)  /* rules (upload/delete) and filter */ 
#define DEB_LOCK          (1UL << 4)  /* locking */ 
#define DEB_HIJACK        (1UL << 5)  /* hijacking */ 
#define DEB_INIT          (1UL << 6)  /* init and cleanup */ 
#define DEB_STRUCTS       (1UL << 7)  /* structs as syscall parameters */ 
#define DEB_VARS          (1UL << 8)  /* variables (VT_PROC_ANCESTORS, etc) */ 
#define DEB_ANY           (~0UL)      /* anything */ 


#endif /* MODULE_INTERFACE_H_ */
