/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il      */
/* License: GNU General Public License                              */

/*
 *$Id: sct_compat.h,v 1.3 2002/09/09 23:55:43 mulix Exp $
 */

#ifndef SCT_COMPAT_H_
#define SCT_COMPAT_H_

#include <linux/version.h>
#include <linux/wait.h>
#include <linux/sched.h> 

/* get the platform definitions */
#include "../sct_rules/archdep.h"

/* default value for sysctl_namelen_t */ 
#define sysctl_namelen_t int

/* ugly hackery to support both 2.2 style get_module_symbol and 2.4 */
#if defined (KERNEL_22)

/* DON'T mess with these - they are required to allow proper expansion of */
/* (possibly) versioned symbols into strings.  -                          */
#define do_turn_into_versioned_string(x) #x                            
#define versioned_string(x) do_turn_into_versioned_string(x)

/* 2.2 doesn't have module owner field in struct file_operations */ 
#define MODULE_OWNER(x) 

#ifndef SET_MODULE_OWNER
# define SET_MODULE_OWNER(a)
#endif /* SET_MODULE_OWNER */ 

#ifndef init_waitqueue_head
# define init_waitqueue_head(x) init_waitqueue((x))
#endif /* init_waitqueue_head */ 

#ifndef wait_queue_head_t
#define wait_queue_head_t struct wait_queue*
#endif /* wait_queue_head_t */

#if defined (KERNEL_22) && LINUX_VERSION_CODE >= KERNEL_VERSION(2,2,19)
#undef sysctl_namelen_t
#define sysctl_namelen_t unsigned int 
#endif /* KERNEL_22 && LINUX_VERSION_CODE >= KERNEL_VERSION(2,2,19) */ 

#elif defined (KERNEL_24) /* we're running 2.4. good */

#define versioned_string(x) (x)
#undef get_module_symbol 
#define get_module_symbol(x,y) (y)

/* module owner field in struct file_operations */ 
#define MODULE_OWNER(x) owner: x,

#endif /* KERNEL_24 */ 

/* fix for current->p_pptr -> current->parent */ 
#if LINUX_VERSION_CODE >= (KERNEL_VERSION(2,5,7))
inline static struct task_struct* task_parent(struct task_struct* p)
{
	return p->parent; 
}
#else 
inline static struct task_struct* task_parent(struct task_struct* p)
{
	return p->p_pptr; 
}
#endif /* KERNEL_VERSION_CODE >= 2.5.7 */ 

/* modules debugging support */
/* if debugging is enabled, artifically raise the priority  */
/* of the logging info. elevate KERN_NOTICE to '1' to we get  */
/* a log even if klogd is run with 'klogd -c 3' */
#ifdef SCT_MODULE_DEBUG

/* values from include/linux/kernel.h */

#undef KERN_NOTICE
#define KERN_NOTICE "<1>"        /* KERN_ALERT */

#undef KERN_INFO 
#define KERN_INFO "<5>"          /* KERN_NOTICE */

#undef KERN_DEBUG
#define KERN_DEBUG "<5>"         /* KERN_NOTICE */

#endif /* SCT_MODULE_DEBUG */

/* are the MODULE_XXX symbols known? */
#ifndef MODULE_AUTHOR
#define MODULE_AUTHOR(x)
#endif

#ifndef MODULE_DESCRIPTION
#define MODULE_DESCRIPTION(x)
#endif

#ifndef MODULE_LICENSE
#define MODULE_LICENSE(x)
#endif

#endif /* SCT_COMPAT_H_ */
