/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2003 guy keren, choo@actcom.co.il             */
/* License: GNU General Public License                              */

/*
 * $Id: util.h,v 1.4 2003/01/27 17:20:43 mulix Exp $
 */ 

#ifndef UTIL_H_
#define UTIL_H_

/* set (*kernel_str) to point to a kernel space copy of the    */
/* string 'str'. return 0 on success, negative errno on error. */ 
int getstring(const char* str, char** kern_str);

/* free a string gotten from getstring */
#define putstring(name)   kfree((name))

/* see module_main.c */ 
extern unsigned long debug_level; 

struct file; 
int debug_write_proc(struct file *file, const char *buffer,
		     unsigned long count, void *data); 

#ifdef SCT_MODULE_DEBUG

#define SCTDBG(level, msg, args...) do {                                  \
        if ((debug_level & level))                                        \
                printk(/* FIXME: level here */                            \
                       "[%s] " msg , __FUNCTION__ , ##args );             \
} while (0)

static inline int debug_is_set(unsigned int level)
{
	return (debug_level & level); 
}

#else /* !defined(SCT_MODULE_DEBUG) */ 

#define SCTDBG(level, msg, args...) do { } while (0)

static inline int debug_is_set(unsigned int level)
{
	return 0; 
}

#endif /* SCT_MODULE_DEBUG */ 

#endif /* UTIL_H_ */
