/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il      */
/* License: GNU General Public License                              */

/*
 * $Id: special_hijack.c,v 1.5 2002/12/08 21:46:40 mulix Exp $
 */

/* 
 * This file has the hijacking functions for special system calls, such 
 * as exit(), execve() or fork(). 
 */ 

#include "linux/sched.h"

/* exit, syscall 1 */ 
static asmlinkage long stub_exit(int error_code)
{
	typedef int (*stub_exit)(int error_code); 
	typedef int (*stub_before_exit)(int error_code); 
	stub_exit orig_syscall_func;
	stub_before_exit before_func;
	struct syscall_hijack_data* hdata = &hijack_data[1];

	atomic_inc(&hdata->invocations); 

	down(&hdata->sem);
	before_func = (stub_before_exit)hdata->before_hijack_func;
	orig_syscall_func = (stub_exit)hdata->orig_syscall;
	up(&hdata->sem);

	if (before_func)
		(*before_func)(error_code);

	down(&hdata->sem);
	if (atomic_read(&hdata->invocations) == 1 && hdata->defered_release_flag == 1) {
	    /* we couldn't reduce the module's use count when the syscall  */
	    /* was originally released (as marked by the 'defered' flag) - */
	    /* so we do so now.                                            */
	    hdata->defered_release_flag = 0;
	    MOD_DEC_USE_COUNT;
	}
	up(&hdata->sem);
	/* WE HAVE A RACE FROM HERE ON - we're still executing on behalf of the stub_exit
	 * code, but no longer protected by the invocation_ref_count. Since the original 
	 * syscall never returns, I don't see a way to avoid this race (except for disallowing  
	 * unloading the module, which we already do)
	 */

	atomic_dec(&hdata->invocations); 
	 /* finally, call the syscall itself */ 
	if (orig_syscall_func)
		(*orig_syscall_func)(error_code); 

	/* exit() must not return! */ 
	BUG(); 
	
	/* doesn't mean anything, silence gcc */ 
	return 0; 
}

/* bdflush, syscall 134 */ 
static asmlinkage long stub_bdflush(int func, long data)
{
	typedef int (*stub_bdflush)(int func, long data);
	typedef int (*stub_before_bdflush)(int func, long data);
	stub_bdflush orig_syscall_func;
	stub_before_bdflush before_func;
	struct syscall_hijack_data* hdata = &hijack_data[134];

	atomic_inc(&hdata->invocations); 

	down(&hdata->sem);
	before_func = (stub_before_bdflush)hdata->before_hijack_func;
	orig_syscall_func = (stub_bdflush)hdata->orig_syscall;
	up(&hdata->sem);

	if (before_func)
		(*before_func)(func, data);

	down(&hdata->sem);
	if (atomic_read(&hdata->invocations) == 1 && hdata->defered_release_flag == 1) {
	    /* we couldn't reduce the module's use count when the syscall  */
	    /* was originally released (as marked by the 'defered' flag) - */
	    /* so we do so now.                                            */
	    hdata->defered_release_flag = 0;
	    MOD_DEC_USE_COUNT;
	}
	up(&hdata->sem);

	/* WE HAVE A RACE FROM HERE ON - we're still executing on behalf of the stub_exit
	 * code, but no longer protected by the invocation_ref_count. Since the original 
	 * syscall never returns, I don't see a way to avoid this race (except for disallowing  
	 * unloading the module, which we already do)
	 */
	atomic_dec(&hdata->invocations); 
	 /* finally, call the syscall itself */ 
	if (orig_syscall_func)
		(*orig_syscall_func)(func, data); 

	/* exit() must not return! */ 
	BUG(); 
	
	/* doesn't mean anything, silence gcc */ 
	return 0; 
}

void init_hijack_syscalls(void)
{
	stub_functions[1] = (func_ptr)stub_exit; 
 	stub_functions[2] = (func_ptr)stub_syscall_fork;
 	stub_functions[11] = (func_ptr)stub_syscall_execve; 
	stub_functions[134] = (func_ptr)stub_bdflush; 
	stub_functions[190] = (func_ptr)stub_syscall_vfork; 
}
