/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il      */
/* License: GNU General Public License                              */

/*
 * $Id: device_file.c,v 1.2 2002/09/07 06:55:50 mulix Exp $
 */ 

/* This code was initially based upon code from the book "Linux Device */
/* Drivers" by Alessandro Rubini and Jonathan Corbet, published by */
/* O'Reilly & Associates.*/

#include <linux/config.h>
#include <linux/version.h>      /* module things */
#include <linux/module.h>       /* module things */
#include <linux/kernel.h>       /* printk() */
#include <asm/uaccess.h>

#include "common/sct_compat.h"

#include "device_file.h"
#include "ctrl_dev.h"
#include "log_dev.h"

/* the device struct */
sct_dev_t sct_dev;

/* forward declarations */

/* utility functions */ 
int init_dev_file(void)
{
	int retval = 0; 

	memset(&sct_dev, 0, sizeof(sct_dev)); 

	if ((retval = init_ctrl_dev_file(&sct_dev.ctrl)) < 0)
		goto done; 

	if ((retval = init_log_dev_file(&sct_dev.log)) < 0)
		goto destroy_ctrl; 
	
	return 0; 

 destroy_ctrl:
	destroy_ctrl_dev_file(sct_dev.ctrl); 	

 done: 
	return retval; 
}

void destroy_dev_file(void)
{
	destroy_log_dev_file(sct_dev.log); 
	sct_dev.log = NULL; 

	destroy_ctrl_dev_file(sct_dev.ctrl); 
	sct_dev.ctrl = NULL; 
}

/* we can't use copy_to/from_user, as they're macros, so we wrap them */ 
unsigned long sct_cp_to_user(void *to, const void *from, unsigned long n)
{
	return copy_to_user(to, from, n); 
}

unsigned long sct_cp_from_user(void *to, const void *from, unsigned long n)
{
	return copy_from_user(to, from, n); 
}


