%{
use Data::Dumper;
%}

%token IDENT INTEGER EOL STRING HEXINT OCTINT
%left LOGICAL_OR
%left LOGICAL_AND
%left BITOR BITXOR
%left BITAND
%left EQ NE
%left LT GT GE LE
%left LSHIFT RSHIFT
%left '+' '-'
%left '*' '/'
%right BITNOT
%left '['
%left DOT

%%
start:	|
		start input
	;

input:		expr EOL	{ my $d = Data::Dumper->new([$1], ["\$value"]); print $d->Dump(); }
	|	EOL
	;

ident: IDENT       { $$ = cons1('ident', $1); }
     | ident '[' expr ']' { $$ = cons2('[]', $1, $3); } 
     | ident DOT ident {  $$ = cons2('member', $1, $3); }
     ;

expr:    ident 	        { $$ = $1; }
    |    INTEGER		{ $$ = cons1("int", $1); }
    |    HEXINT         { $$ = cons1("int", hex($1)); }
    |    OCTINT         { $$ = cons1("int", oct($1)); }
    |    STRING         { $$ = cons1("string", expand_backslash($1)); }
    |   BITNOT expr { $$ = cons1("~", $2); }
    |   expr LT expr { $$ = cons2("<", $1, $3 ); }
    |   expr LE expr { $$ = cons2("<=", $1, $3 ); }
    |   expr GE expr { $$ = cons2(">=", $1, $3 ); }
    |   expr GT expr { $$ = cons2(">", $1, $3 ); }
    |   expr NE expr { $$ = cons2("!=", $1, $3 ); }
    |   expr EQ expr { $$ = cons2("==", $1, $3 ); }
    |   expr LSHIFT expr { $$ = cons2("<<", $1, $3); }
    |   expr RSHIFT expr { $$ = cons2(">>", $1, $3); }    
	|	expr '+' expr	{ $$ = cons2('+', $1, $3); }
	|	expr '-' expr	{ $$ = cons2('-', $1, $3); }
	|	expr '*' expr	{ $$ = cons2('*', $1, $3); }
	|	expr '/' expr	{ $$ = cons2('/', $1, $3); }
    |   expr LOGICAL_AND expr { $$ = cons2("&&", $1, $3); }
    |   expr LOGICAL_OR expr { $$ = cons2("||", $1, $3); }    
    |   expr BITAND expr { $$ = cons2("&", $1, $3); }
    |   expr BITOR expr { $$ = cons2("|", $1, $3); }
    |   expr BITXOR expr { $$ = cons2("^", $1, $3); }
	|	'(' expr ')'	{ $$ = $2; }
	;
%%
# $Id: sct_parser.y,v 1.7 2002/07/17 15:12:13 shlomif Exp $

sub expand_backslash
{
    $_ = shift;
    s/^\"//;
    s/\"$//;
    s/\\n/\n/g;
    s/\\r/\r/g;
    s/\\\\/\\/g;
    s/\\x[A-Fa-f0-9]({,2})/chr(hex($1))/ge;
    s/\\([0-7]{1,3})/chr(oct($1))/ge;
    s/\\(.)/$1/g;
    return $_;
}

sub cons1
{
    my ($op, $arg) = @_;
    return { 'op' => $op, 'arg' => $arg };
}

sub cons2
{
    my ($op, $left,$right) = @_;
    return { 'op' => $op, 'l' => $left, 'r' => $right };
}

sub yyerror {
    my ($msg, $s) = @_;
    die "$msg at " . $s->name . " line " . $s->lineno . ".\n";
}
