/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2003 Muli Ben-Yehuda, mulix@mulix.org         */ 
/* License: GNU General Public License                              */

/* 
 * $Id: parselib.cpp,v 1.2 2003/02/01 19:21:22 mulix Exp $
 */

#include <string> 

#include <sys/utsname.h> 

#include "tokens_to_types.h"
#include "syscalls.h"

/* you have to call this before beginning to use the parselib */ 
/* returns 0 on success */
std::string build_syscalls_dat_location(const char* base)
{
	std::string str(base); 
	
	struct utsname uts; 
	memset(&uts, 0, sizeof(uts)); 

	if (uname(&uts) < 0) {
		perror("uname"); 
		/* fallback to plain 'syscalls.dat' */ 
		return str;
	}

	str += "-"; 
	str += uts.release; 
	return str; 
}

int sct_parselib_init(void)
{
	tokens::init(); 

	std::string syscalls; 

	syscalls = build_syscalls_dat_location(SYSCALLS_DAT_LOCATION_BASE); 

	if (!syscalls::load(syscalls.c_str()))
		return 1; 

	return 0; 
}
