/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il	    */
/* License: GNU Lesser General Public License                       */

/*
 *$Id: arch-linux-kernel.h,v 1.14 2003/01/28 13:48:50 mulix Exp $
 */

#ifndef ARCH_LINUX_KERNEL_H_
#define ARCH_LINUX_KERNEL_H_

#ifndef LINUX_KERNEL
#error "arch-linux-kernel.h included and LINUX_KERNEL not defined!"
#endif

/* malloc/free support */
# include <linux/kernel.h>
# include <linux/slab.h>
# include <linux/vmalloc.h>
# include <linux/types.h>
# include <linux/stddef.h>
# include <linux/string.h>
# include <linux/kernel.h>
# include <linux/ctype.h>
# include <linux/types.h>
# include <linux/version.h>
# include <linux/sched.h>

#define mem_alloc(foo)  kmalloc((foo), GFP_KERNEL)
#define mem_free(foo)   kfree((foo))
#define vmem_alloc(foo) vmalloc((foo))
#define vmem_free(foo)  vfree((foo))

/* generic print macro */
# define PPRINT(fmt, args...) printk(fmt, ## args)

# ifdef SCT_RULES_DEBUG 

/* values from include/linux/kernel.h */

#undef KERN_NOTICE
#define KERN_NOTICE "<1>"        /* KERN_ALERT */

#undef KERN_INFO 
#define KERN_INFO "<5>"          /* KERN_NOTICE */

#undef KERN_DEBUG
#define KERN_DEBUG "<5>"         /* KERN_NOTICE */

# endif /* SCT_RULES_DEBUG */

/* assert in kernel space */

#undef assert
#define assert(x) do {if (!(x)) BUG(); } while (0)

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
#define KERNEL_24
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,2,0)
#define KERNEL_22
#else 
#error This is neither linux 2.4 no linux 2.2. What is it?
#endif 

#include <asm/page.h>
#ifndef BUG
#define BUG() do { \
	printk("kernel BUG at %s:%d!\n", __FILE__, __LINE__); \
} while (0)
#endif

#if defined (KERNEL_22)

#include <asm/semaphore.h>
#ifndef DECLARE_MUTEX
# define DECLARE_MUTEX(name)			struct semaphore name=MUTEX
#endif
#ifndef init_MUTEX
# define init_MUTEX(x)                           *(x)=MUTEX
#endif

#endif /* KERNEL_22 */

typedef struct semaphore sct_sem_t;

static inline pid_t sct_getpid(void)
{
	return current->pid; 
}

static inline int sct_kill(pid_t pid, int sig)
{
	/* TODO: what does the priv field mean exactly? */ 
	return kill_proc(pid, sig, 1); 
}

static inline void sct_suspend(pid_t pid)
{
	kill_proc(pid, SIGSTOP, 1); 
}

/* loguser lock and unlock functions */
static inline int sct_lock_sem_interruptible(struct semaphore* sem)
{
	return down_interruptible(sem);
}

static inline void sct_lock_sem(struct semaphore* sem)
{
	down(sem); 
}

static inline void sct_unlock_sem(struct semaphore* sem)
{
	up(sem);
}

static inline void sct_sema_init(struct semaphore* sem, int val)
{
	sema_init (sem, val);
}


#endif /* ARCH_LINUX_KERNEL_H_ */
