/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 guy keren, choo@actcom.co.il	            */
/* License: GNU Lesser General Public License                       */

/*
 *$Id: encode_decode_public.h,v 1.8 2003/01/31 14:08:45 mulix Exp $
 */

#ifndef _ENCODE_DECODE_PUBLIC_H
#define _ENCODE_DECODE_PUBLIC_H

/*
 * Rule engine - primitive types encoding/decoding functions.
 */

struct sct_membuf; 

/* encode the given unsigned char at the given pos, */
/* as long as the buffer is not too small.          */
int encode_uchar(struct sct_membuf* m, unsigned char c);

/* decode the given unsigned char at the given pos, */
/* as long as we are not past end of buf.           */
int decode_uchar(struct sct_membuf* m, unsigned char* pc);

/* encode the given signed char at the given pos, */
/* as long as the buffer is not too small.          */
int encode_char(struct sct_membuf* m, char c);

/* decode the given signed char at the given pos, */
/* as long as we are not past end of buf.           */
int decode_char(struct sct_membuf* m, char* pc);

/* encode the given int at the given pos,  */
/* as long as the buffer is not too small. */
int encode_int(struct sct_membuf* m, int i);

/* decode the given int at the given pos, */
/* as long as we are not past end of buf. */
int decode_int(struct sct_membuf* m, int* pi);

/* encode the given unsigned int at the given pos, */
/* as long as the buffer is not too small.         */
int encode_uint(struct sct_membuf* m, unsigned int i);

/* decode the given unsigned int at the given pos, */
/* as long as we are not past end of buf.          */
int decode_uint(struct sct_membuf* m, unsigned int* pi);

/* encode the given long at the given pos, */
/* as long as the buffer is not too small. */
int encode_long(struct sct_membuf* m, long l);

/* decode the given long at the given pos, */
/* as long aswe are not past end of buf.   */
int decode_long(struct sct_membuf* m, long* pl);

/* encode the given unsigned long at the given pos, */
/* as long as the buffer is not too small.          */
int encode_ulong(struct sct_membuf* m, unsigned long l);

/* decode the given unsigned long at the given pos, */
/* as long aswe are not past end of buf.            */
int decode_ulong(struct sct_membuf* m, unsigned long* pl);

/* encode the given long long at the given pos, */
/* as long as the buffer is not too small. */
int encode_long_long(struct sct_membuf* m, long long l);

/* decode the given long long at the given pos, */
/* as long long aswe are not past end of buf.   */
int decode_long_long(struct sct_membuf* m, long long* pl);

/* encode the given short at the given pos, */
/* as long as the buffer is not too small.  */
int encode_short(struct sct_membuf* m, short s);

/* decode the given short at the given pos, */
/* as long as we are not past end of buf.   */
int decode_short(struct sct_membuf* m, short* ps);

/* encode the given unsigned short at the given pos, */
/* as long as the buffer is not too small.           */
int encode_ushort(struct sct_membuf* m, unsigned short s);

/* decode the given unsigned short at the given pos, */
/* as long as we are not past end of buf.            */
int decode_ushort(struct sct_membuf* m, unsigned short* ps);

/* encode the given float at the given pos, */
/* as long as the buffer is not too small.  */
int encode_float(struct sct_membuf* m, float f);

/* decode the given float at the given pos, */
/* as long as we are nto past end of buf.   */
int decode_float(struct sct_membuf* m, float* pf);

/* encode the given string at the given pos, */
/* as long as the buffer is not too small.   */
int encode_string(struct sct_membuf* m, const char* str);

/* decode the given string at the given pos, */
/* as long as we are not past end of buf.    */
int decode_string(struct sct_membuf* m, char** pstr);

/* encode the given buffer 'in' at the given pos, */
/* as long as the destination buffer 'out' is not too small. */
int encode_buffer(struct sct_membuf* m, const void* in, unsigned int size); 

/* decode the given buffer '*pdst' at the given pos, */
/* as long as we are not past end of the buf 'src'.  */
int decode_buffer(struct sct_membuf* m, void** pdst, unsigned int* psize); 

/* encode the given ptr at the given pos, */
/* as long as the buffer is not too small.          */
int encode_ptr(struct sct_membuf* m, void* ptr);

/* decode the given ptr at the given pos, */
/* as long as we are not past end of buf.   */
int decode_ptr(struct sct_membuf* m, void** pp);

#endif /* _ENCODE_DECODE_PUBLIC_H */
