#!/usr/bin/env python2
#
# this script runs the gcalc executable with one rule loaded at a time.
# It then checks that gcalc has launched correctly and is running.
#
# written to solve the 'gcalc crash' bug and is very, very hackish, so
# don't bother telling me about how hackish it is. I'll clean it up and
# generalize it eventually. Patches are welcome, of course.
# 
# Muli. 
import os
import string
import time

def run_command(com):
    """ run a command in the local directory """
    print "running cmd: %s" % (string.join(com))
    rc = os.spawnvp(os.P_WAIT, com[0], com) 
    if (rc != 0):
        print com
        error = string.join(com) + " failed"
        raise error

def exec_command(com):
    """ run a comand in this process """
    print "execing cmd: %s" % (string.join(com))
    os.execvp(com[0], com)

class gcalc:
    """ wrapper around the gcalc process """

    def run(self, timeout = 2):
        pid = os.fork()
        if (pid == 0):
            # child
            exec_command(['gcalc'])
            print "eek, gcalc process returned!"
            sys.exit(1)
        else:
            time.sleep(timeout)
            if (not self.exists()):
                return 1
            
            os.kill(pid, 15)
            time.sleep(0) # give the signal time to be delivered 
            if (self.exists()):
                raise "could not kill gcalc"

            return 0

    def exists(self):
        """ check if a pid is running by checking the output of ps, or kill """
        f = os.popen('ps -ax | grep gcalc')
        lines = f.readlines()
        for l in lines:
            fields = string.splitfields(l)
            if (fields[4] == "gcalc"): 
                f.close()
                return 1

        f.close()
        return 0

class rule_file:
    """ a class representing a file with syscalltrack rules """
    rule_file_name = "./app_test_rule_file"
    fd = 0

    def __init__(self, name = rule_file_name):
        self.fd = open(name, "w");

    def write(self, str):
        self.fd.write(str)
        self.fd.flush()

    def close(self): 
        self.fd.close()

    def load_rule_file(self):
        """ load a rule file """
        run_command(["sct_config", "upload", self.rule_file_name])

class rule:
    """ create and upload a syscalltrack rule """

    syscall = ''

    def __init__(self, syscall):
        self.syscall = syscall

    def append_rule_to_file(self, file):
	when = ''
	if (self.syscall == "exit" or self.syscall == "execve"):
		when = "before" 
	else: 
		when = "after" 
        rule_str = """
        rule {
	    syscall_name =  %s 
            rule_name = rule_x
            when = %s
	    filter_expression { COMM ~= \"gcalc\" }
	    action { type=LOG }
        }

        """ % (self.syscall, when)

        file.write(rule_str)

    def load(self, file):
        """ create and load a syscalltrack rule """ 
        self.append_rule_to_file(file)
        file.load_rule_file()
   

def get_syscalls():
    """ get the list of syscall names from syscalls.list """
    f = open('syscalls.list')
    syscalls = []
    for line in f.readlines():
        if line[0] == '#':
            continue 
        syscalls.append(string.strip(line))

    return syscalls 

def main():
    """ the main function """
    file = rule_file()
    
    for syscall in get_syscalls(): 
        r = rule(syscall)
        g = gcalc()

        r.load(file)
        if (g.run(5)):
            error = "!!!syscall '%s' caused gcalc to fail to run!!!" % (syscall)
            raise error

if (__name__ == "__main__"):
    main()
