#!/usr/bin/perl -W

# Copyright (C) 2003 Orna Agmon <agmon@tx.technion.ac.il>
# License: GNU General Public License

# $Id: reg.pl,v 1.3 2003/02/01 19:21:22 mulix Exp $
# reg.pl takes the output of tester and compares it to a previous
# output, stored in .previous. If no such file exists, .previous is
# created from current input.  If the test is repeatable, then the
# output is compared word for word.  The output is a report of the
# number of erred lines and the number of erred tests.

use strict;

my @status_prev_tests=stat('.previous');
my $first=0;
if (!defined($status_prev_tests[7]) or ($status_prev_tests[7] ==0))
{				#size of .previous
    print "First time- using these test results for comparison,".
	" saving as .previous\n";
    $first=1;
}



if ($first)
{
    open (PREV,">.previous") or die "could not open .previous for write\n";
} else {
    open (PREV,"<.previous") or die "could not open .previous for read\n";
}

my ($in,$prev);    
my $bad=0;
my $bad_tests=0;

while ($in=<>)
{
    unless ($first) {
	$prev=<PREV>;
	
	#the test name must be the same
	($prev eq $in) or 
	    die "$prev differs from  \n$in";
    }
    
    ($in=~/^starting test (\'[^\']+\')[^\[]+\[(\w*)\]/) or
	die "could not find test name in '$in'";
    
    my $testname=$1;
    my $repeatable=$2;
    $repeatable=($repeatable=~/^REPEATABLE$/);
    
    #get results for this test
    my $inresults=$in;
    my $prev_bad=$bad;
    while ($inresults!~/test $testname succeeded\n$/)
    {
	defined($in=<>) or 
	    die "Input ends prematurely for test $testname";
	$inresults.=$in;
	unless ($first){
	    (defined($prev=<PREV>)) or
		die ".previous ends prematurely for test $testname";
	    if (($repeatable) and ($in ne $prev))
	    {
		$bad++;
		print "Input differs: old line, new line:\n$prev$in";
	    }
	}
    }
    if ($first)
    {
	print PREV $inresults;
    } else {
	if ($prev_bad!=$bad)
	{
	    print "Failed:";
	    if ($repeatable) {
		print "repeatable ";
	    } else {
		print "non-repeatable ";
	    }
	    print "$testname\n";
	    $bad_tests++;
	    
	}
    }
}
unless ($first){
    my $extra=<PREV>;
    
    (defined($extra)) and  
	die ".previous is longer than current. extra line $extra";
}
close(PREV);
if ($bad)
{
    die "$bad errors were found in $bad_tests different tests";
} else {
    print "Input passed all tests\n";
}
