#!/usr/bin/env python2
# 
# A simple script to do all of the tests before we release
#

import os
import sys
import re

# define this to 1 to run the apptest on each iteratoion
# yes, we need a command line flag. I'm lazy. 
RUN_APPTEST = 1

class SpawnError(Exception):   # the exception we throw if any error occurs
    def __init__(self, value): # constructor
        self.value = value
    def __str__(self):         # string representation
        return `self.value`


def run_command(command):
    """ run a command in the local directory """
    if (command == None): return
    com = command.split(); 
    rc = os.spawnvp(os.P_WAIT, com[0], com); 
    if (rc != 0):
        raise SpawnError, command + " failed"

def run_apptest():
    """ run the apptest script """
    os.chdir("apptest")
    run_command("./app_test.py")
    os.chdir("..")

def run_tester():
    """ run the tester exeuctable """
    # now, run tester
    os.chdir("stress")
    run_command("./tester")
    os.chdir("..")


def run_correctness():
    """ run the correctness tests script """
    # run the tests
    os.chdir("correctness")
    run_command("./correctness.pl")
    os.chdir("..")


def check_for_root():
    if (os.getuid() != 0):
        print("sorry, must be root to run these tests")
        sys.exit(1)


def check_module_loaded():
    regexps = [re.compile("sct_hijack"), re.compile("sct_rules")]
    match = 0
  
    for l in os.popen("/sbin/lsmod").readlines():
        for r in regexps:
            if r.match(l):
              match = match + 1
              # we need two matches
        if (match == 2):
            return
    print("sorry, but the modules do not appear to be loaded?")
    sys.exit(1)


def main():
    check_for_root()
    check_module_loaded()

    num_iterations = 10
    while num_iterations > 0:
        try: 
            if (RUN_APPTEST):
                run_apptest()
            run_tester()
            run_correctness()
            num_iterations = num_iterations - 1
        except KeyboardInterrupt:
            print "ok, stopping now"
            sys.exit(0)


if __name__ == "__main__":
    main()
