/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2003 Muli Ben-Yehuda, mulix@mulix.org         */
/* License: GNU General Public License                              */

/* 
 * $Id: ancestors.c,v 1.1 2003/01/31 18:35:32 mulix Exp $
 */

#include <stdio.h>
#include <unistd.h>
#include <sys/syscall.h>

#include "sct_rules/rule_engine_public.h"
#include "sct_rules/filter_vars_public.h"

#include "sct_ctrl_lib/sct_ctrl_lib.h" 

#include "tester.h"

/* lives in tester.c and guaranteed to be initialized by */ 
/* the time we are called */ 
extern sct_ctrl_t ctrl; 

int test_ancestors_simple(void)
{
	int rc; 
	sct_filter_node_t* ancestors; 
	sct_filter_node_t* match; 
	sct_filter_node_t* ppid = sct_make_int_val(getppid()); 
	sct_action_t* act = sct_make_action_log(4, NULL); 
	const sct_rule_when_t when = RW_BEFORE; 
	const int syscall_id = __NR_getpid; 

	const int rule_id = -1; 

	ancestors = sct_make_var_val(VT_PROC_ANCESTORS, DT_MULTI); 

	/* build the match here */ 
	match = sct_make_in_match(ppid, ancestors); 

	CHECK1(ancestors != NULL); 
	CHECK1(act != NULL); 
	CHECK1(ppid != NULL); 

	rc = sct_ctrl_add_rule(ctrl, rule_id, when, syscall_id,  
			       match, act, 1); 
	CHECK1(rc >= 0); 

	printf("loaded a rule for getpid(), to check if %d is "
	       "in our ancestors\n", getppid()); 
		
	sleep(3); 

	(void)getpid(); 

	printf("finished with getpid, moving on\n"); 

	sleep(3); 

	return 1; 
}
