/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il      */
/* License: GNU General Public License                              */

/* 
 * $Id: logging_test.c,v 1.1 2003/01/31 18:35:32 mulix Exp $
 */ 

/* test the module's logging mechanism */ 

#include "sct_rules/rule_engine_public.h"
#include "sct_rules/rule_engine_private.h"
#include "sct_rules/filter_vars_public.h"
#include "sct_rules/membuf.h"
#include "sct_rules/loguser.h" 

#include "tester.h"

/* variable calculation callbacks. */
sct_filter_node_t* proc_current_callback(sct_var_type_t type, void *data) {
	sct_filter_node_t *f = NULL;
    
	switch (type) {
	case VT_PROC_UID:
		f = sct_make_long_val(1);
		break;
	case VT_PROC_EUID:
		f = sct_make_long_val(2);
		break;
	case VT_PROC_SUID:
		f = sct_make_long_val(3);
		break;
	case VT_PROC_GID:
		f = sct_make_long_val(4);
		break;
	case VT_PROC_EGID:
		f = sct_make_long_val(5);
		break;
	case VT_PROC_SGID:
		f = sct_make_long_val(6);
		break;
	case VT_PROC_PID:
		f = sct_make_long_val(7);
		break;
	case VT_PROC_COMM:
		f = sct_make_string_val("blabla");
		break;
	default:
		f = NULL;
		break;
	}
	return f;
}

static void init_vars(void)
{
	reset_all_vars();
	set_persistent_callback(VT_PROC_UID, proc_current_callback, NULL);
	set_persistent_callback(VT_PROC_EUID, proc_current_callback, NULL);
	set_persistent_callback(VT_PROC_SUID, proc_current_callback, NULL);
	set_persistent_callback(VT_PROC_GID, proc_current_callback, NULL);
	set_persistent_callback(VT_PROC_EGID, proc_current_callback, NULL);
	set_persistent_callback(VT_PROC_SGID, proc_current_callback, NULL);
	set_persistent_callback(VT_PROC_PID, proc_current_callback, NULL);
	set_persistent_callback(VT_PROC_COMM, proc_current_callback, NULL);
	set_persistent_callback(VT_PROC_PPID, proc_current_callback, NULL);
	set_persistent_callback(VT_PROC_PCOMM, proc_current_callback, NULL);
	set_persistent_callback(VT_PROC_ANCESTORS, proc_current_callback, NULL);
}

int test_logging(void)
{
	init_vars(); 
	tracker_do_logging(NULL, 
			   1, 
			   2, 
			   "foo", 
			   "syscall: %pid[%comm]: %sid_%sname() (rule %ruleid)\n",
			   "", 
			   ""); 
	reset_all_vars(); 
	return 1; 
}

int test_logging_cyclic(void)
{
	int i; 
	struct sct_membuf* log = sct_membuf_create(29, NULL, NULL, 
					      SCT_MEMBUF_DROP_OLD); 

	sct_loguser_t* loguser = sct_loguser_create(log, NULL, NULL);
	unsigned char* data; 
	unsigned char* print; 
	char tst[] = "]: 2_foo() (rule 1)\\10[\"blabla\\0"; 
	init_vars(); 
	
	if (!log || !loguser)
		return 0; 

	i = 2; 
	while (i--) {
		tracker_do_logging(loguser, 1, 2, "foo", 
				   "syscall: %pid[%comm]: %sid_%sname() (rule %ruleid)\n",
				   "", ""); 
	}

	data = ((unsigned char*)sct_membuf_get_data(log)); 
	data[sct_membuf_total_size(log)-1] = '\0'; 

	print = sct_buffer_to_string(data, sct_membuf_total_size(log)); 
	CHECK(strcmp(print, tst), 0); 

	mem_free(print); 
	sct_membuf_destroy(log); 
	sct_loguser_destroy(loguser); 

	reset_all_vars(); 
	
	return 1; 
}
