/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@mulix.org         */
/* License: GNU General Public License                              */

/* 
 * $Id: tester.h,v 1.1 2003/01/31 18:35:32 mulix Exp $
 */ 

#ifndef TESTER_H_
#define TESTER_H_

#include <stdio.h>

#include "tests.h" 

/* if <> 0, we will be a little more lenient and not dump core */
/* immediately on errors */ 
extern int debug; 

void fatal(const char* func, const char* file,
	   unsigned int line, const char* fmt, ...);

void logerror(const char* func, const char* file,
	      unsigned int line, const char* fmt, ...);

#define die(format, args...) \
	fatal(__FUNCTION__,__FILE__,__LINE__, format , ## args)
#define logerr(format, args...) \
	logerror(__FUNCTION__,__FILE__,__LINE__, format , ## args)

#define CHECK2(__check, __expect) do {                                 \
	int __res = __check; /* eval only once */                     \
        if (__expect != __res) {                                      \
                if (debug)                                            \
	                logerr("CHECK FAILED: %s returned %d, "       \
			      "expected %d\n", #__check , __res,      \
			      (int)__expect);                         \
	        else                                                  \
			die("CHECK FAILED: %s returned %d, "          \
			    "expected %d\n", #__check , __res,        \
			    (int) __expect);                          \
        }                                                             \
} while (0)                                                           

#define CHECK1(__check) CHECK2(__check, 1)

#define CHECK(a, b) CHECK2(a, b)

#endif /* TESTER_H_ */
