/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2003 Muli Ben-Yehuda, mulix@mulix.org         */ 
/* License: GNU General Public License                              */

/* 
 * $Id: tests.c,v 1.2 2003/02/01 10:04:26 orna Exp $
 */ 

#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <stdio.h>

#include "tests.h" 

#define MAKE_TEST(name, flags) { #name, (flags), test_##name }

struct test tests[] = {
	MAKE_TEST(all_syscalls_register_rules, SCT_SUBSYS_MODULE), 
	MAKE_TEST(simple_syscall_usage, SCT_SUBSYS_MODULE),
	MAKE_TEST(rule_register_deregister, SCT_SUBSYS_MODULE), 
	MAKE_TEST(all_syscalls_usage, SCT_SUBSYS_MODULE), 
	MAKE_TEST(kill_suspend_process, SCT_SUBSYS_MODULE), 
	MAKE_TEST(loguser, SCT_SUBSYS_MODULE), 
	MAKE_TEST(ancestors_simple, SCT_SUBSYS_MODULE), 
	MAKE_TEST(filter_eval_bit, SCT_SUBSYS_RULES | SCT_TEST_REPEATABLE), 
	MAKE_TEST(eval_complex_filter, SCT_SUBSYS_RULES ),
	MAKE_TEST(filter_serialize_deserialize, SCT_SUBSYS_RULES | SCT_TEST_REPEATABLE),
	MAKE_TEST(rules_serialize_deserialize, SCT_SUBSYS_RULES | SCT_TEST_REPEATABLE),
	MAKE_TEST(tracking_rule_add_delete, SCT_SUBSYS_RULES | SCT_TEST_REPEATABLE),
	MAKE_TEST(tracking_rule_add_delete_empty_list, SCT_SUBSYS_RULES | SCT_TEST_REPEATABLE),
	MAKE_TEST(indexed_field_variables, SCT_SUBSYS_RULES | SCT_TEST_REPEATABLE),
	MAKE_TEST(kernel_variables, SCT_SUBSYS_RULES | SCT_TEST_REPEATABLE),
	MAKE_TEST(short_circuiting, SCT_SUBSYS_RULES | SCT_TEST_REPEATABLE),
	MAKE_TEST(filter_node_vectors, SCT_SUBSYS_RULES | SCT_TEST_REPEATABLE),
	MAKE_TEST(filter_node_in_match, SCT_SUBSYS_RULES | SCT_TEST_REPEATABLE),
	MAKE_TEST(membuf, SCT_SUBSYS_RULES | SCT_TEST_REPEATABLE),
	MAKE_TEST(tracking_rule_list, SCT_SUBSYS_RULES | SCT_TEST_REPEATABLE),
	MAKE_TEST(logging, SCT_SUBSYS_RULES | SCT_TEST_REPEATABLE),
	MAKE_TEST(get_rules, SCT_SUBSYS_RULES | SCT_SUBSYS_MODULE | SCT_TEST_REPEATABLE),
	MAKE_TEST(void_ptr_buffer, SCT_SUBSYS_RULES ),
	MAKE_TEST(buffer_match, SCT_SUBSYS_RULES | SCT_TEST_REPEATABLE),
	MAKE_TEST(membuf_enlarge, SCT_SUBSYS_RULES | SCT_TEST_REPEATABLE),
	MAKE_TEST(membuf_resize, SCT_SUBSYS_RULES | SCT_TEST_REPEATABLE),
	MAKE_TEST(ring_membuf_exact, SCT_SUBSYS_RULES | SCT_TEST_REPEATABLE),
	MAKE_TEST(ring_membuf_drop_new, SCT_SUBSYS_RULES | SCT_TEST_REPEATABLE),
	MAKE_TEST(ring_membuf_drop_old, SCT_SUBSYS_RULES | SCT_TEST_REPEATABLE),
	MAKE_TEST(cyclic_membuf, SCT_SUBSYS_RULES | SCT_TEST_REPEATABLE),
	MAKE_TEST(logging_cyclic, SCT_SUBSYS_RULES | SCT_TEST_REPEATABLE),
	MAKE_TEST(struct, SCT_SUBSYS_RULES | SCT_TEST_REPEATABLE),
	MAKE_TEST(long_long, SCT_SUBSYS_RULES | SCT_TEST_REPEATABLE),
	MAKE_TEST(void_ptr, SCT_SUBSYS_RULES | SCT_TEST_REPEATABLE),
	MAKE_TEST(vector_kernel_vars, SCT_SUBSYS_RULES | SCT_TEST_REPEATABLE),
	MAKE_TEST(serialize_vectors, SCT_SUBSYS_RULES | SCT_TEST_REPEATABLE),
	{ NULL, 0, NULL } 
}; 

static inline size_t min(int a, int b)
{
	return (a < b ? a : b); 
}

/* returns the number of tests found */ 
unsigned int find_test(const char* name, struct test** test)
{
	struct test* p; 
	const size_t nlen = strlen(name); 
	unsigned int found = 0; 
	
	for (p = &tests[0]; p && p->test_fn; ++p) {
		size_t tlen = strlen(p->name); 

		size_t len = min(nlen, tlen); 

		if (!strncmp(p->name, name, len)) {
			*test = p; 
			++found; 
		}
	}

	if (found != 1)
		*test = NULL; 

	return found; 
}

const char* subsystem_as_str(unsigned int subsys)
{
	/* if we add more subsstems, rewrite me */ 
	if (subsys == (SCT_SUBSYS_RULES | SCT_SUBSYS_MODULE))
		return "module, rules"; 
	
	if (subsys == SCT_SUBSYS_RULES)
		return "rules"; 

	if (subsys == SCT_SUBSYS_MODULE)
		return "module"; 

	return "unknown"; 
}

unsigned int str_to_subsystem(const char* str)
{
	assert(str); 
	
	if (!strcmp(str, "module"))
		return SCT_SUBSYS_MODULE; 

	if (!strcmp(str, "rules"))
		return SCT_SUBSYS_RULES; 

	return 0; 
}

void list_tests(FILE* f)
{
	struct test* p; 
	
	for (p = &tests[0]; p && p->test_fn; ++p) {
		unsigned int subsys; 
		const char* subsys_str; 

		subsys = (p->flags & SCT_SUBSYS_BITS); 
		subsys_str = subsystem_as_str(subsys); 

		fprintf(f, "%s (%s)\n", p->name, subsys_str); 
	}
}
