/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il	    */
/* License: GNU General Public License                              */

/*
 * $Id: sctdbg.cpp,v 1.6 2003/01/27 17:21:10 mulix Exp $
 */

#include <string>
#include <iostream>
#include <fstream>
#include <map>
#include <algorithm>
#include <iterator>

#include <errno.h>

extern "C" {
#include "module/common/module_interface.h" 
}

#include "sct_parselib/sstreamwrap.h" 

typedef std::map<const std::string, unsigned long> dbg_map; 

static void make_map(dbg_map& m)
{
	m.clear(); 
	m["none"] =     DEB_NONE; 
	m["info"] =     DEB_INFO; 
	m["dev_file"] = DEB_DEV_FILE; 
	m["sysctl"] =   DEB_SYSCTL; 
	m["rules"] =    DEB_RULES; 
	m["lock"] =     DEB_LOCK; 
	m["hijack"] =   DEB_HIJACK; 
	m["init"] =     DEB_INIT; 
	m["structs"] =  DEB_STRUCTS; 
	m["vars"] =     DEB_VARS; 
	m["any"] =      DEB_ANY; 
}


static char ctolower(const char& c)
{
	return tolower(c); 
}

static std::string downcase(const std::string& s)
{
	sstreamwrap d; 
	transform(s.begin(), s.end(), 
		  std::ostream_iterator<char>(d), ctolower); 
	return d.str(); 
}

int main(int argc, char* argv[])
{	
	dbg_map m; 
	make_map(m); 
	unsigned long dbgflag = 0; 
	int i = 0; 

	while (i++ < argc) {
		if (!argv[i])
			continue; 
		
		std::string val = argv[i]; 
		if (m.find(downcase(val)) != m.end())
			dbgflag |= m[downcase(val)];
	}

	std::cout << dbgflag << std::endl; 
	return EXIT_SUCCESS; 
}
